@@ -54,12 +54,17 @@
 using namespace hy_global;
 using namespace hy_env;
 
+//#define _UBER_VERBOSE_MX_UPDATE_DUMP
+//#define _UBER_VERBOSE_MX_UPDATE_DUMP_EVAL 1
+
 //_______________________________________________________________________________________________
 
 _CalcNode::_CalcNode    () {
     theProbs = nil;    // default constructor, doesn't do much
     compExp = nil;
     matrixCache = nil;
+    flags = 0;
+    cBase = 0;
 }
 
 //_______________________________________________________________________________________________
@@ -84,6 +89,7 @@ void    _CalcNode::InitializeCN     ( _String const& parms, int, _VariableContai
     if (theIndex < 0) return;
 
     cBase         = 0;
+    flags         = 0;
     theProbs      = nil;
     compExp       = nil;
     matrixCache   = nil;
@@ -166,6 +172,7 @@ long      _CalcNode::SetDependance (long var_index) {
             it needs to be marked as such in this CalcNode
          */
         
+ 
         PopulateAndSort ([&] (_AVLList& avl) -> void {
             LocateVar (var_index)->ScanForVariables (avl,true);
         }).Each ( [&] (long var_index, unsigned long idx) -> void {
@@ -177,10 +184,10 @@ long      _CalcNode::SetDependance (long var_index) {
         // also clear out previously computed matrix exponentials
         if (compExp) {
             DeleteAndZeroObject(compExp);
-        } else {
+        } /*else {
             if (matrixCache) {
             }
-        }
+        }*/
     }
     return var_index;
 }
@@ -191,7 +198,7 @@ void    _CalcNode::SetCodeBase (int codeBase) {
     if (codeBase>0) {
         if (codeBase != cBase || !theProbs) {
             if (theProbs) {
-                delete theProbs;
+                delete [] theProbs;
             }
             theProbs = new hyFloat [codeBase];
             cBase = codeBase;
@@ -308,7 +315,7 @@ _String*            _CalcNode::GetBranchSpec (void) {
 
 //__________________________________________________________________________________
 
-void _CalcNode::ReplaceModel (_String& modelName, _VariableContainer* parent_tree) {
+void _CalcNode::ReplaceModel (_String& modelName, _VariableContainer* parent_tree, _AVLListXL* aCache, bool clean_locals) {
     // TODO: SLKP 20171203 this is FUGLY
   RemoveModel    ();
 
@@ -330,8 +337,10 @@ void _CalcNode::ReplaceModel (_String& modelName, _VariableContainer* parent_tre
 
   long my_var_index = theIndex;
 
-  DeleteVariable (theIndex, false); // this will clean up all the node.xx variables
-  InitializeCN   (modelName, 0 , parent_tree);
+  if (clean_locals) {
+      DeleteVariable (theIndex, false); // this will clean up all the node.xx variables
+  }
+  InitializeCN   (modelName, 0 , parent_tree, aCache);
 
   if (container_object) {
     _Variable * new_node = LocateVar(my_var_index);
@@ -464,25 +473,33 @@ bool        _CalcNode::NeedNewCategoryExponential(long catID) const {
 }
 
 //_______________________________________________________________________________________________
-bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix* storeRateMatrix, _List* queue, _SimpleList* tags, _List* bufferedOps)
+bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix* storeRateMatrix, _List* queue, _SimpleList* tags, _List* bufferedOps, bool direct_copy)
 {
     // assumed that NeedToExponentiate was called prior to this function
 
     //_Variable* curVar, *locVar;
+    bool    isExplicitForm  = HasExplicitFormModel ();
+
+    if (direct_copy && bufferedOps && bufferedOps->countitems() == 1 && isExplicitForm) {
+        SetCompExp ((_Matrix*)bufferedOps->GetItem(0)->makeDynamic(), totalCategs>1?categID:-1);
+        return false;
+    }
     
     _SimpleList * var_lists [2] = {iVariables, dVariables};
     for (_SimpleList* iterable : var_lists) {
         ForEachLocalVariable(iterable,CopyModelParameterValue);
     }
   
+    /*
     #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
-      if (1|| likeFuncEvalCallCount == _UBER_VERBOSE_MX_UPDATE_DUMP_LF_EVAL && gVariables) {
+     // if (1|| likeFuncEvalCallCount == _UBER_VERBOSE_MX_UPDATE_DUMP_LF_EVAL && gVariables) {
         for (unsigned long i=0; i<gVariables->lLength; i++) {
           _Variable* curVar = LocateVar(gVariables->GetElement(i));
-          fprintf (stderr, "[_CalcNode::RecomputeMatrix] Node %s, var %s, value = %15.12g\n", GetName()->sData, curVar->GetName()->sData, curVar->Compute()->Value());
+          fprintf (stderr, "[_CalcNode::RecomputeMatrix] Node %s, var %s, value = %15.12g\n", GetName()->get_str(), curVar->GetName()->get_str(), curVar->Compute()->Value());
         }
-      }
+      //}
     #endif
+    */
 
     /*
     for (unsigned long i=0; i<categoryVariables.lLength; i++) {
@@ -496,31 +513,33 @@ bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix
 
     if (!storeRateMatrix) {
       if (totalCategs>1) {
-  #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
-        fprintf (stderr, "[_CalcNode::RecomputeMatrix] Deleting category %ld for node %s at %p\n", categID, GetName()->sData, GetCompExp(categID));
-  #endif
-        DeleteObject(GetCompExp(categID, true));
-
+         if (clear_exponentials()) {
+            #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
+                  fprintf (stderr, "[_CalcNode::RecomputeMatrix] Deleting category %ld for node %s at %p\n", categID, GetName()->get_str(), GetCompExp(categID));
+            #endif
+           DeleteObject(GetCompExp(categID, true));
+        }
       } else {
         if (compExp) {
-            DeleteAndZeroObject(compExp);
+            if (clear_exponentials ()) {
+                DeleteAndZeroObject(compExp);
+            }
         }
       }
     }
 
-    bool    isExplicitForm  = HasExplicitFormModel ();
-
+ 
     if (isExplicitForm && bufferedOps) {
-        _Matrix * bufferedExp = (_Matrix*)GetExplicitFormModel()->Compute (0,nil, bufferedOps);
+        _Matrix * bufferedExp = (_Matrix*)GetExplicitFormModel(map_global_to_local_category (categID))->Compute (0,nil, bufferedOps);
         #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
-            fprintf (stderr, "[_CalcNode::RecomputeMatrix] Setting (buffered) category %ld/%ld for node %s\n", categID, totalCategs, GetName()->sData);
+            fprintf (stderr, "[_CalcNode::RecomputeMatrix] Setting (buffered) category %ld/%ld for node %s\n", categID, totalCategs, GetName()->get_str());
          #endif
         SetCompExp ((_Matrix*)bufferedExp->makeDynamic(), totalCategs>1?categID:-1);
         return false;
     }
 
     unsigned long      previous_length = queue && tags ? queue->lLength: 0;
-    _Matrix * myModelMatrix = GetModelMatrix(queue,tags);
+    _Matrix * myModelMatrix = GetModelMatrix(queue,tags,map_global_to_local_category(categID));
 
     if (isExplicitForm && !myModelMatrix) { // matrix exponentiations got cached
         if (queue && queue->lLength > previous_length) {
@@ -531,12 +550,12 @@ bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix
         }
     } else {
 
-        if (myModelMatrix->MatrixType()!=_POLYNOMIAL_TYPE) {
+        if (!myModelMatrix->is_polynomial()) {
             _Matrix *temp = nil;
             if (isExplicitForm) {
-                temp = (_Matrix*)myModelMatrix->makeDynamic();
+                temp = (_Matrix*)myModelMatrix;
             } else {
-                temp = (_Matrix*)myModelMatrix->MultByFreqs(theModel);
+                temp = (_Matrix*)myModelMatrix->MultByFreqs(theModel, true);
             }
             
             // copy updated model (local) constrained parameters to their external references
@@ -546,7 +565,7 @@ bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix
                     if (!model_var -> IsIndependent()) {
                         _Variable * param_var = LocateVar (var_idx);
                         if (param_var->IsIndependent()) {
-                            param_var->SetValue(param_var->Compute());
+                            param_var->SetValue(param_var->Compute(),true,true,NULL);
                         }
                     }
                 }
@@ -560,17 +579,25 @@ bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix
 
 
             if (queue) {
+                
                 (*queue) << temp;
                 if (tags) {
-                    (*tags) << (isExplicitForm);
+                    (*tags) << (isExplicitForm ? (queue->countitems() - previous_length == 1 ? -1 : 1) : 0);
                 }
                 return isExplicitForm;
             }
 
             #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
-                fprintf (stderr, "[_CalcNode::RecomputeMatrix] Setting category %ld/%ld for node %s\n", categID, totalCategs, GetName()->sData);
+                fprintf (stderr, "[_CalcNode::RecomputeMatrix] Setting category %ld/%ld for node %s\n", categID, totalCategs, GetName()->get_str());
             #endif
-            SetCompExp ((_Matrix*)(isExplicitForm?temp:temp->Exponentiate(1., true)), totalCategs>1?categID:-1);
+            
+            if (isExplicitForm) {
+                temp->AddAReference();
+                SetCompExp (temp, totalCategs>1?categID:-1);
+            } else {
+                SetCompExp (temp->Exponentiate(1., true), totalCategs>1?categID:-1);
+            }
+            
 
         } else {
             compExp = (_Matrix*)myModelMatrix->Evaluate(false);
@@ -578,16 +605,34 @@ bool        _CalcNode::RecomputeMatrix  (long categID, long totalCategs, _Matrix
     }
     return false;
 }
+//_______________________________________________________________________________________________
+
+long        _CalcNode::map_global_to_local_category (long cat) const {
+    if (remapMyCategories.lLength) return remapMyCategories.list_data[cat*(categoryVariables.lLength+1)];
+    return cat;
+}
 
 //_______________________________________________________________________________________________
-void        _CalcNode::SetCompExp  (_Matrix* m, long catID) {
-    compExp = m;
+void        _CalcNode::SetCompExp  (_Matrix* m, long catID, bool do_exponentiation) {
+    
+    _Matrix ** store_exp_here;
+    
     if (catID >= 0 && matrixCache) {
         if (remapMyCategories.lLength) {
             catID = remapMyCategories.list_data[catID*(categoryVariables.lLength+1)];
         }
-        matrixCache[catID] = compExp;
+        store_exp_here = &(matrixCache[catID]);
+    } else {
+        store_exp_here = &compExp;
     }
+    
+    if (do_exponentiation) {
+        compExp = m->Exponentiate(1., true, *store_exp_here);
+        reuse_exponentials ();
+    } else {
+        compExp = m;
+    }
+    *store_exp_here = compExp;
 }
 //_______________________________________________________________________________________________
 _Matrix*        _CalcNode::ComputeModelMatrix  (bool) {
@@ -619,7 +664,9 @@ _Matrix*    _CalcNode::GetCompExp       (long catID, bool doClear) const {
         _Matrix* ret = matrixCache?matrixCache[catID]:compExp;
 
         if (doClear && matrixCache) {
-            matrixCache[catID] = nil;
+            if (matrixCache[catID] && matrixCache[catID]->CanFreeMe()) {
+                matrixCache[catID] = nil;
+            } 
         }
         return ret;
     }
@@ -634,6 +681,7 @@ BaseRef     _CalcNode::makeDynamic(void) const {
     res->categoryIndexVars.Duplicate (&categoryIndexVars);
     res->theValue = theValue;
     res->cBase = cBase;
+    res->flags = flags;
     if (cBase) {
         res->theProbs = new hyFloat [cBase];
         CopyArray(res->theProbs, theProbs, cBase);
@@ -713,7 +761,7 @@ void     _CalcNode::SetupCategoryMap (_List& containerVariables, _SimpleList& cl
         }
     }
     
-    //printf ("Node remap at %s yielded %s\n", GetName()->sData, _String((_String*)remapMyCategories.toStr()).sData);
+    //printf ("Node remap at %s yielded %s (%s)\n", GetName()->get_str(), _String((_String*)remapMyCategories.toStr()).get_str(),  _String((_String*)classCounter.toStr()).get_str());
     
 }
 
@@ -731,10 +779,17 @@ node<long>* _CalcNode::LocateMeInTree (void) const {
 
 //_______________________________________________________________________________________________
 
-void _CalcNode::ConvertToSimpleMatrix (void) const {
+void _CalcNode::ConvertToSimpleMatrix (unsigned long category_count) {
     _Formula * mf = GetExplicitFormModel();
     if (mf) {
-        mf->ConvertMatrixArgumentsToSimpleOrComplexForm (false);
+         if (!templateFormulaClone) {
+            templateFormulaClone = new _Formula* [category_count];
+            for (long i = 0; i < category_count; i++) {
+                templateFormulaClone[i] = new _Formula (*mf);
+                templateFormulaClone[i]->ConvertMatrixArgumentsToSimpleOrComplexForm (false);
+            }
+        }
+        
     } else {
         _Matrix * mm [2] = {GetModelMatrix(), GetFreqMatrix()};
         for (_Matrix * m : mm) {
@@ -747,10 +802,20 @@ void _CalcNode::ConvertToSimpleMatrix (void) const {
 
 //_______________________________________________________________________________________________
 
-void _CalcNode::ConvertFromSimpleMatrix (void) {
+void _CalcNode::ConvertFromSimpleMatrix (unsigned long category_count) {
     _Formula * mf = GetExplicitFormModel();
-    if (mf) {
-        mf->ConvertMatrixArgumentsToSimpleOrComplexForm (true);
+    if (templateFormulaClone || mf) {
+        if (templateFormulaClone) {
+            //printf ("_CalcNode::ConvertFromSimpleMatrix %s => %d\n", GetName()->get_str(), category_count);
+            for (long i = 0; i < category_count; i++) {
+                delete templateFormulaClone[i];
+            }
+            delete [] templateFormulaClone;
+            templateFormulaClone = nil;
+            GetExplicitFormModel()->ConvertMatrixArgumentsToSimpleOrComplexForm(true);
+        } else {
+            mf->ConvertMatrixArgumentsToSimpleOrComplexForm(true);
+        }
     } else {
         _Matrix * mm [2] = {GetModelMatrix(), GetFreqMatrix()};
         for (_Matrix * m : mm) {
